package esercitazione4.binarysearchtrees;

public class BinarySearchTree {
	
	private Integer value;
	private BinarySearchTree leftChild;
	private BinarySearchTree rightChild;
	private BinarySearchTree parent;
	
	public BinarySearchTree(Integer value) {
		this.value = value;
		this.parent = null;
		this.leftChild = null;
		this.rightChild = null;
	}
	
	public Integer getValue() {
		return value;
	}

	public BinarySearchTree getLeftChild() {
		return leftChild;
	}

	public BinarySearchTree getRightChild() {
		return rightChild;
	}

	public BinarySearchTree getParent() {
		return parent;
	}

	
	public Boolean isLeftChild(){
		return (this.value < parent.value);
	}
	
	public Boolean isRightChild(){
		return (this.value > parent.value);
	}
	
	public Boolean isLeaf(){
		return ((this.leftChild == null) && (this.rightChild == null));
	}
	
	public Boolean hasOnlyLeftChild(){
		return ((this.leftChild != null) && (this.rightChild == null));
	}
	
	public Boolean hasOnlyRightChild(){
		return ((this.leftChild == null) && (this.rightChild != null));
	}
	
	public BinarySearchTree insert(Integer i) {
	
		BinarySearchTree nodoInserito = null;
		
		if (this.getValue() > i) {
			if (this.leftChild != null)
				nodoInserito = this.leftChild.insert(i);
			else {
				nodoInserito = new BinarySearchTree(i);
				nodoInserito.parent = this;
				this.leftChild = nodoInserito;
			}
		}
		if (this.getValue() < i) {
			if (this.rightChild != null)
				nodoInserito = this.rightChild.insert(i);
			else {
				nodoInserito = new BinarySearchTree(i);
				nodoInserito.parent = this;
				this.rightChild = nodoInserito;
			}
		}
		
		return nodoInserito;
		
	}
	
	
	
	public BinarySearchTree find(Integer i) {
		BinarySearchTree nodoTrovato = null;
		
		if (this.getValue().equals(i))
			nodoTrovato = this;
		else {
			if ((this.getValue() > i) && (this.leftChild != null))
				nodoTrovato = this.leftChild.find(i);
			if ((this.getValue() < i) && (this.rightChild != null))
				nodoTrovato = this.rightChild.find(i);
		}
		
		return nodoTrovato;
		
	}
	
	
	
	public Integer min() {
		BinarySearchTree m = this;
		
		while (m.leftChild != null)
			m = m.leftChild;
		
		return m.getValue();
	}
	
	public void printInDescendingOrder(){
	
		if (this.rightChild != null)
			this.rightChild.printInDescendingOrder();
		
		System.out.print(this.value + " ");
		
		if (this.leftChild != null)
			this.leftChild.printInDescendingOrder();
	}
	
	
	public BinarySearchTree delete(Integer i) { 
	
		BinarySearchTree elementoDaCancellare = this.find(i);	
		BinarySearchTree elementoDaAggangiare = null;
		
		if (elementoDaCancellare != null) {
			
			if (elementoDaCancellare.isLeaf()) 
				elementoDaAggangiare = null;
			else if (elementoDaCancellare.hasOnlyLeftChild())
				elementoDaAggangiare = elementoDaCancellare.leftChild;
			else if (elementoDaCancellare.hasOnlyRightChild())
				elementoDaAggangiare = elementoDaCancellare.rightChild;
			else {
				// TODO: Da completare per esercizio
			}
			
			
			if (elementoDaCancellare.isLeftChild())
				elementoDaCancellare.parent.leftChild = elementoDaAggangiare;
			else
				elementoDaCancellare.parent.rightChild = elementoDaAggangiare;
		}
		
		return elementoDaCancellare;
		
	}
	
}
